'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.InStorageCache = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _apolloCacheInmemory = require('apollo-cache-inmemory');

var _objectStorageCache = require('./objectStorageCache');

var _transform = require('./transform');

var _utils = require('./utils');

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var defaults = {
  normalize: _utils.normalize,
  denormalize: _utils.denormalize
};

var InStorageCache = function (_InMemoryCache) {
  _inherits(InStorageCache, _InMemoryCache);

  /**
   * @property {(Object|Function)} storage - The Storage to use.
   *
   * @see https://www.w3.org/TR/webstorage/#storage
   */


  /**
   * @property {Function} [normalize] - Normalization callback. Executed
   * prior to storing a resource.
   */


  /**
   * @property {Function} [denormalize] - Denormalization callback. Executed
   * after retrieving a cached resource from the storage.
   */


  /**
   * @property {Function} [shouldPersist] - Callback to determine if a given
   * data should be cached.
   */


  /**
   * @property {boolean} [addPersistField] - Whether or not should add a
   * __persist field to all non scalar types in a query.
   */
  function InStorageCache() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var storage = _ref.storage,
        _ref$normalize = _ref.normalize,
        normalize = _ref$normalize === undefined ? defaults.normalize : _ref$normalize,
        _ref$denormalize = _ref.denormalize,
        denormalize = _ref$denormalize === undefined ? defaults.denormalize : _ref$denormalize,
        _ref$shouldPersist = _ref.shouldPersist,
        shouldPersist = _ref$shouldPersist === undefined ? function () {
      return true;
    } : _ref$shouldPersist,
        _ref$addPersistField = _ref.addPersistField,
        addPersistField = _ref$addPersistField === undefined ? false : _ref$addPersistField,
        _ref$prefix = _ref.prefix,
        prefix = _ref$prefix === undefined ? '' : _ref$prefix,
        config = _objectWithoutProperties(_ref, ['storage', 'normalize', 'denormalize', 'shouldPersist', 'addPersistField', 'prefix']);

    _classCallCheck(this, InStorageCache);

    var _this = _possibleConstructorReturn(this, (InStorageCache.__proto__ || Object.getPrototypeOf(InStorageCache)).call(this, config));

    if (!storage) {
      throw new _utils.InStorageCacheError('You must provide a storage to use');
    }

    if (!(0, _utils.validStorage)(storage)) {
      throw new _utils.InStorageCacheError('You must provide a valid storage to use');
    }

    _this.addPersistField = typeof addPersistField === 'function' ? addPersistField : function () {
      return addPersistField;
    };

    _this.persistence = {
      storage: storage,
      normalize: normalize,
      denormalize: denormalize,
      shouldPersist: shouldPersist,
      prefix: prefix
    };

    _this.data = new _objectStorageCache.ObjectStorageCache(null, _this.persistence);
    return _this;
  }

  /**
   * @property {string} [prefix] - The prefix to use when saving to the
   * provided storage. Useful to diff items from this and other persisting
   * systems that share the same storage.
   */


  _createClass(InStorageCache, [{
    key: 'transformDocument',
    value: function transformDocument(doc) {
      return _get(InStorageCache.prototype.__proto__ || Object.getPrototypeOf(InStorageCache.prototype), 'transformDocument', this).call(this, this.addPersistField(doc) ? (0, _transform.addPersistFieldToDocument)(doc) : doc);
    }
  }]);

  return InStorageCache;
}(_apolloCacheInmemory.InMemoryCache);

exports.InStorageCache = InStorageCache;