'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createHasLocalDirective = exports.hasLocalDirective = undefined;

var _graphql = require('graphql');

var extractDirectives = function extractDirectives(originalQuery, directive) {
  var hasDirective = false;

  var query = (0, _graphql.visit)(originalQuery, {
    Directive: function Directive(_ref) {
      var name = _ref.name.value;

      if (name === directive) {
        hasDirective = true;
        return null;
      }
    }
  });

  return { query: query, hasDirective: hasDirective };
};

var createHasLocalDirective = function createHasLocalDirective(directive) {
  return function (operation) {
    var mutate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

    var _extractDirectives = extractDirectives(operation.query, directive),
        query = _extractDirectives.query,
        hasDirective = _extractDirectives.hasDirective;

    if (mutate) {
      operation.query = query;
    }

    return hasDirective;
  };
};

/**
 * Checks if a given operation uses @local directive.
 *
 * @param {Object} operation The GraphQL operation.
 * @param {boolean} [mutate] Wheter or not the operation's query should
 *  get it's @local directives extracted.
 */
var hasLocalDirective = createHasLocalDirective('local');

exports.hasLocalDirective = hasLocalDirective;
exports.createHasLocalDirective = createHasLocalDirective;