'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createLocalCacheLink = exports.LocalCacheLink = undefined;

var _apolloLink = require('apollo-link');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LocalCacheLinkError = function (_Error) {
  _inherits(LocalCacheLinkError, _Error);

  function LocalCacheLinkError(message) {
    _classCallCheck(this, LocalCacheLinkError);

    return _possibleConstructorReturn(this, (LocalCacheLinkError.__proto__ || Object.getPrototypeOf(LocalCacheLinkError)).call(this, '[LocalCacheLink] ' + message));
  }

  return LocalCacheLinkError;
}(Error);

/**
 * Checks if the the storage is valid.
 */


var validStorage = function validStorage(storage) {
  return storage && storage.getItem && storage.setItem && storage.removeItem && storage.clear;
};

var LocalCacheLink = function (_ApolloLink) {
  _inherits(LocalCacheLink, _ApolloLink);

  /**
   * @property {(boolean|Function)} [shouldCache] - The cache policy. When set
   *  to `true` defaults to cache all. When set to a callback, will receive the
   *  operation as argument, and should return `true` or `false`. Defaults to `true`.
   */


  /**
   * @property {Function} [generateKey] - A callback to generate a key for
   *  an operation. Defaults to using `operation.toKey`.
   */


  /**
   * @property {(Object|Function)} [storage] - The Storage in use. Will
   *  default to window.localStorage when available.
   *
   * @see https://www.w3.org/TR/webstorage/#storage
   */


  /**
   * @property {Function} [normalize] - Normalization callback. Executed
   * prior to storing a query result.
   */
  function LocalCacheLink() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref$shouldCache = _ref.shouldCache,
        shouldCache = _ref$shouldCache === undefined ? true : _ref$shouldCache,
        _ref$generateKey = _ref.generateKey,
        generateKey = _ref$generateKey === undefined ? function (operation) {
      return operation.toKey();
    } : _ref$generateKey,
        _ref$storage = _ref.storage,
        storage = _ref$storage === undefined ? typeof localStorage !== 'undefined' ? localStorage : null : _ref$storage,
        _ref$normalize = _ref.normalize,
        normalize = _ref$normalize === undefined ? function (data) {
      return JSON.stringify(data);
    } : _ref$normalize,
        _ref$denormalize = _ref.denormalize,
        denormalize = _ref$denormalize === undefined ? function (data) {
      return JSON.parse(data);
    } : _ref$denormalize;

    _classCallCheck(this, LocalCacheLink);

    var _this2 = _possibleConstructorReturn(this, (LocalCacheLink.__proto__ || Object.getPrototypeOf(LocalCacheLink)).call(this));

    _initialiseProps.call(_this2);

    if (!storage && typeof localStorage === 'undefined') {
      throw new LocalCacheLinkError('Could not determine a storage to use');
    }

    _this2.generateKey = generateKey;
    _this2.shouldCache = shouldCache;
    _this2.storage = storage;
    _this2.normalize = normalize;
    _this2.denormalize = denormalize;
    return _this2;
  }

  /**
   * Determines if an operation is cacheable or not.
   */


  /**
   * @property {Function} [denormalize] - Denormalization callback. Executed
   * after retrieving a cached query result from the store.
   */


  /**
   * Retrieves the storage.
   */


  /**
   * Link query requester.
   */


  return LocalCacheLink;
}(_apolloLink.ApolloLink);

var _initialiseProps = function _initialiseProps() {
  var _this3 = this;

  this.isCacheable = function (operation) {
    return typeof _this3.shouldCache === 'function' ? _this3.shouldCache(operation) : _this3.shouldCache;
  };

  this.getStorage = function (operation) {
    var storage = typeof _this3.storage === 'function' ? _this3.storage(operation) : _this3.storage;

    if (!validStorage(storage)) {
      throw new LocalCacheLinkError('Must provide a valid storage');
    }

    return storage;
  };

  this.request = function (operation, forward) {
    if (_this3.isCacheable(operation)) {
      var key = _this3.generateKey(operation);
      var cached = _this3.getStorage(operation).getItem(key);

      if (cached) {
        return _apolloLink.Observable.of(_this3.denormalize(cached, operation));
      }

      return forward(operation).map(function (result) {
        _this3.getStorage(operation).setItem(key, _this3.normalize(result, operation));

        return result;
      });
    }

    return forward(operation);
  };
};

var createLocalCacheLink = function createLocalCacheLink(config) {
  return new LocalCacheLink(config);
};

exports.LocalCacheLink = LocalCacheLink;
exports.createLocalCacheLink = createLocalCacheLink;