'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ObjectStorageCache = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _utils = require('./utils');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ObjectStorageCache = function () {
  function ObjectStorageCache() {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var persistence = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, ObjectStorageCache);

    if (!persistence.storage) {
      throw new _utils.InStorageCacheError('You must provide a persistence.storage to use');
    }

    if (!(0, _utils.validStorage)(persistence.storage)) {
      throw new _utils.InStorageCacheError('You must provide a valid persistence.storage to use');
    }

    if (typeof persistence.normalize !== 'function') {
      throw new _utils.InStorageCacheError('You must provide a persistence.normalize function');
    }

    if (typeof persistence.denormalize !== 'function') {
      throw new _utils.InStorageCacheError('You must provide a persistence.denormalize function');
    }

    if (typeof persistence.shouldPersist !== 'function') {
      throw new _utils.InStorageCacheError('You must provide a persistence.shouldPersist function');
    }

    if (typeof persistence.prefix !== 'string') {
      throw new _utils.InStorageCacheError('You must provide a persistence.prefix string');
    }

    this.persistence = persistence;
    this.data = _extends({}, data);
  }

  _createClass(ObjectStorageCache, [{
    key: 'toObject',
    value: function toObject() {
      var persisted = (0, _utils.toObject)(this.persistence.storage, this.persistence.denormalize, this.persistence.prefix);

      return _extends({}, persisted, this.data);
    }
  }, {
    key: 'get',
    value: function get(dataId) {
      if (!this.data[dataId] && this.persistence.shouldPersist('get', dataId)) {
        this.data[dataId] = this.persistence.denormalize(this.persistence.storage.getItem('' + this.persistence.prefix + dataId));
      }

      return this.data[dataId];
    }
  }, {
    key: 'set',
    value: function set(dataId, value) {
      if (this.persistence.shouldPersist('set', dataId, value)) {
        this.persistence.storage.setItem('' + this.persistence.prefix + dataId, this.persistence.normalize(value));
      }

      this.data[dataId] = value;
    }
  }, {
    key: 'delete',
    value: function _delete(dataId) {
      if (this.persistence.shouldPersist('delete', dataId)) {
        this.persistence.storage.removeItem('' + this.persistence.prefix + dataId);
      }

      this.data[dataId] = undefined;
    }
  }, {
    key: 'clear',
    value: function clear() {
      this.persistence.storage.clear();
      this.data = {};
    }
  }, {
    key: 'replace',
    value: function replace(newData) {
      this.data = {};

      for (var dataId in newData) {
        this.set(dataId, _extends({}, this.get(dataId), newData[dataId]));
      }
    }
  }]);

  return ObjectStorageCache;
}();

exports.ObjectStorageCache = ObjectStorageCache;