'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createPersistLink = exports.PersistLink = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _graphql = require('graphql');

var _apolloLink = require('apollo-link');

var _traverse = require('traverse');

var _traverse2 = _interopRequireDefault(_traverse);

var _transform = require('./transform');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Given a data result object path, return the equivalent query selection path.
 *
 * @param {Array} path The data result object path. i.e.: ["a", 0, "b"]
 * @return {String} the query selection path. i.e.: "a.b"
 */
var toQueryPath = function toQueryPath(path) {
  return path.filter(function (key) {
    return isNaN(Number(key));
  }).join('.');
};

/**
 * Given a data result object, attach __persist values.
 */
var attachPersists = function attachPersists(paths, object) {
  var queryPaths = paths.map(toQueryPath);

  return (0, _traverse2.default)(object).map(function () {
    if (!this.isRoot && this.node && _typeof(this.node) === 'object' && Object.keys(this.node).length && !Array.isArray(this.node)) {
      var path = toQueryPath(this.path);

      this.update(_extends({
        __persist: Boolean(queryPaths.find(function (queryPath) {
          return queryPath.indexOf(path) === 0 || path.indexOf(queryPath) === 0;
        }))
      }, this.node));
    }
  });
};

var PersistLink = function (_ApolloLink) {
  _inherits(PersistLink, _ApolloLink);

  _createClass(PersistLink, null, [{
    key: 'shouldPersist',

    /**
     * InStorageCache shouldPersist implementation for a __persist field validation.
     */
    value: function shouldPersist(op, dataId, data) {
      // console.log(dataId, data)
      return dataId === 'ROOT_QUERY' || !data || !!data.__persist;
    }

    /**
     * InStorageCache addPersistField implementation to check for @perist directives.
     */

  }]);

  function PersistLink() {
    _classCallCheck(this, PersistLink);

    var _this = _possibleConstructorReturn(this, (PersistLink.__proto__ || Object.getPrototypeOf(PersistLink)).call(this));

    _this.request = function (operation, forward) {
      var _extractPersistDirect = (0, _transform.extractPersistDirectivePaths)(operation.query, _this.directive),
          query = _extractPersistDirect.query,
          paths = _extractPersistDirect.paths;

      // Early exit if no persist directive found.


      if (!paths.length) return forward(operation);

      // Replace query with one without @persist directives.
      operation.query = query;

      // Remove requesting __persist fields.
      operation.query = (0, _graphql.visit)(operation.query, {
        Field: function Field(_ref, key, parent, path, ancestors) {
          var name = _ref.name.value;

          if (name === '__persist') {
            return null;
          }
        }
      });

      return forward(operation).map(function (result) {
        if (result.data) {
          result.data = attachPersists(paths, result.data);
        }

        return result;
      });
    };

    _this.directive = 'persist';
    return _this;
  }

  /**
   * Link query requester.
   */


  return PersistLink;
}(_apolloLink.ApolloLink);

PersistLink.addPersistField = function (doc) {
  return (0, _transform.hasPersistDirective)(doc);
};

var createPersistLink = function createPersistLink(config) {
  return new PersistLink(config);
};

exports.PersistLink = PersistLink;
exports.createPersistLink = createPersistLink;