'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var InStorageCacheError = function (_Error) {
  _inherits(InStorageCacheError, _Error);

  function InStorageCacheError(message) {
    var _ref;

    _classCallCheck(this, InStorageCacheError);

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    return _possibleConstructorReturn(this, (_ref = InStorageCacheError.__proto__ || Object.getPrototypeOf(InStorageCacheError)).call.apply(_ref, [this, '[InStorageCacheError] ' + message].concat(args)));
  }

  return InStorageCacheError;
}(Error);

var validStorage = function validStorage(storage) {
  return Boolean(storage && storage.getItem && storage.setItem && storage.removeItem && storage.clear);
};

/**
 * Iterates each key of the storage and execute the callback on it.
 *
 * @param {Object} storage The storage instance.
 * @param {Function} callback The iteration callback.
 */
var iterate = function iterate(storage, callback) {
  for (var i = 0; i < storage.length; ++i) {
    callback(storage.key(i));
  }
};

/**
 * Creates a plain object from all the storage's persisted data.
 *
 * @param {Object} storage The storage instance.
 * @param {Function} denormalize Method of denormalizing the retrieved resource.
 * @param {String} prefix The storage persisting key prefix.
 */
var toObject = function toObject(storage) {
  var denormalize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (value) {
    return value;
  };
  var prefix = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';

  var object = {};

  iterate(storage, function (key) {
    if (key.indexOf(prefix) === 0) {
      object[key.slice(prefix.length)] = denormalize(storage.getItem(key));
    }
  });

  return object;
};

var normalize = JSON.stringify;
var denormalize = JSON.parse;

exports.InStorageCacheError = InStorageCacheError;
exports.validStorage = validStorage;
exports.toObject = toObject;
exports.normalize = normalize;
exports.denormalize = denormalize;